@@ -1,4 +1,5 @@
-print.IPCW <- function(x,digits=3){
+##' @S3method print IPCW
+print.IPCW <- function(x,digits=3,...){
   cat("\nEstimated inverse of the probability of censoring weights (IPCW)\n\n")
   method=switch(x$method,
     "cox"="Cox regression",
